"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Filter = exports.Filterable = void 0;
const orm_1 = require("@spinajs/orm");
const di_1 = require("@spinajs/di");
const http_1 = require("@spinajs/http");
function Filterable(operatorsOrClass, isAggregate) {
    return (0, orm_1.extractDecoratorPropertyDescriptor)((model, _target, propertyKey) => {
        if (model.FilterableColumns === undefined) {
            model.FilterableColumns = new Map();
        }
        if ((0, di_1.isConstructor)(operatorsOrClass)) {
            // if we have @belonsgTo relation we add to this filter schema all avaible filters
            // from the related model
            const descriptor = (0, orm_1.extractModelDescriptor)(operatorsOrClass);
            if (descriptor.FilterableColumns) {
                const keys = [...descriptor.FilterableColumns.keys()];
                const ops = keys.map((k) => {
                    return [`${propertyKey}.${k}`, descriptor.FilterableColumns.get(k)];
                });
                model.FilterableColumns = new Map([...model.FilterableColumns.entries(), ...ops]);
            }
        }
        else {
            if (!operatorsOrClass) {
                throw new Error(`Filterable decorator on ${model.Name} model, property ${propertyKey} must have operators defined`);
            }
            model.FilterableColumns.set(propertyKey, operatorsOrClass);
        }
        const columnDesc = model.Columns.find((c) => c.Name === propertyKey);
        if (!columnDesc) {
            // we dont want to fill all props, they will be loaded from db and mergeg with this
            model.Columns.push((0, orm_1._prepareColumnDesc)({ Name: propertyKey, Aggregate: isAggregate ?? false, Virtual: true }));
        }
        else {
            columnDesc.Aggregate = isAggregate ?? false;
        }
    });
}
exports.Filterable = Filterable;
function Filter(model) {
    return (0, http_1.Route)((0, http_1.Parameter)('FilterModelRouteArg', null, model));
}
exports.Filter = Filter;
//# sourceMappingURL=decorators.js.map