"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MODEL_STATIC_MIXINS = void 0;
const orm_1 = require("@spinajs/orm");
require("./builders.js");
exports.MODEL_STATIC_MIXINS = {
    async filter(filters) {
        const { query } = (0, orm_1.createQuery)(this, orm_1.SelectQueryBuilder);
        return query.filter(filters);
    },
    filterColumns() {
        const modelDescriptor = this.getModelDescriptor();
        if (!modelDescriptor) {
            throw new orm_1.OrmException(`Model ${this.constructor.name} has no descriptor`);
        }
        return [...modelDescriptor.FilterableColumns.entries()].map(([key, val]) => {
            return {
                column: key,
                operators: val,
            };
        });
    },
    filterSchema() {
        const modelDescriptor = this.getModelDescriptor();
        if (!modelDescriptor) {
            throw new orm_1.OrmException(`Model ${this.constructor.name} has no descriptor`);
        }
        return {
            type: 'array',
            items: {
                type: 'object',
                anyOf: [...modelDescriptor.FilterableColumns.entries()].map(([key, val]) => {
                    return {
                        type: 'object',
                        required: ['Column', 'Operator'],
                        properties: {
                            Column: { const: key },
                            Value: { type: ['string', 'integer', 'array'] },
                            Operator: { type: 'string', enum: val },
                        },
                    };
                }),
            },
        };
    },
};
//# sourceMappingURL=model.js.map