import { _prepareColumnDesc, extractDecoratorPropertyDescriptor, extractModelDescriptor } from '@spinajs/orm';
import { isConstructor } from '@spinajs/di';
import { Parameter, Route } from '@spinajs/http';
export function Filterable(operatorsOrClass, isAggregate) {
    return extractDecoratorPropertyDescriptor((model, _target, propertyKey) => {
        if (model.FilterableColumns === undefined) {
            model.FilterableColumns = new Map();
        }
        if (isConstructor(operatorsOrClass)) {
            // if we have @belonsgTo relation we add to this filter schema all avaible filters
            // from the related model
            const descriptor = extractModelDescriptor(operatorsOrClass);
            if (descriptor.FilterableColumns) {
                const keys = [...descriptor.FilterableColumns.keys()];
                const ops = keys.map((k) => {
                    return [`${propertyKey}.${k}`, descriptor.FilterableColumns.get(k)];
                });
                model.FilterableColumns = new Map([...model.FilterableColumns.entries(), ...ops]);
            }
        }
        else {
            if (!operatorsOrClass) {
                throw new Error(`Filterable decorator on ${model.Name} model, property ${propertyKey} must have operators defined`);
            }
            model.FilterableColumns.set(propertyKey, operatorsOrClass);
        }
        const columnDesc = model.Columns.find((c) => c.Name === propertyKey);
        if (!columnDesc) {
            // we dont want to fill all props, they will be loaded from db and mergeg with this
            model.Columns.push(_prepareColumnDesc({ Name: propertyKey, Aggregate: isAggregate ?? false, Virtual: true }));
        }
        else {
            columnDesc.Aggregate = isAggregate ?? false;
        }
    });
}
export function Filter(model) {
    return Route(Parameter('FilterModelRouteArg', null, model));
}
//# sourceMappingURL=decorators.js.map