var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Orm, OrmException, extractModelDescriptor, RelationType, OrmNotFoundException } from '@spinajs/orm';
import { Parameter, Route, ParameterType, ArgHydrator, RouteArgs } from '@spinajs/http';
import { Injectable, Container, Autoinject, Bootstrapper, DI } from '@spinajs/di';
import { MODEL_STATIC_MIXINS } from './model.js';
import { InvalidArgument } from '@spinajs/exceptions';
export * from './interfaces.js';
export * from './model.js';
export * from './decorators.js';
export * from './extension.js';
export * from './route-arg.js';
export * from './builders.js';
export * from './dto.js';
export * from './response-methods/OrmNotFound.js';
let AsDbModel = class AsDbModel extends RouteArgs {
    get SupportedType() {
        return 'AsDbModel';
    }
    async extract(callData, _args, param, req) {
        if (!req.body) {
            return { CallData: callData, Args: null };
        }
        const result = new param.RuntimeType();
        const data = req.body[param.Options.field ?? param.Name];
        if (!data) {
            throw new OrmException(`Cannot hydrate model, field ${param.Options.field ?? param.Name} is required`);
        }
        result.hydrate(data);
        return { CallData: callData, Args: result };
    }
};
AsDbModel = __decorate([
    Injectable()
], AsDbModel);
export { AsDbModel };
let FromDbModel = class FromDbModel extends RouteArgs {
    async resolve() { }
    get SupportedType() {
        return 'FromDB';
    }
    async extract(callData, args, param, req) {
        let result = null;
        if (param?.Options?.query) {
            result = await param.Options.query.call(param.RuntimeType.query(), args, this._extractValue(param, req)).firstOrThrow(new OrmNotFoundException("Resource not found"));
            ;
        }
        else {
            result = await this.fromDbModelDefaultQueryFunction(callData, args, param, req);
        }
        return { CallData: callData, Args: result };
    }
    _extractValue(param, req) {
        let pkValue = null;
        const field = param?.Options?.paramField ?? param.Name;
        switch (param?.Options?.paramType) {
            case ParameterType.FromQuery:
                pkValue = req.query[field];
                break;
            case ParameterType.FromBody:
                pkValue = req.body ? req.body[field] : null;
                break;
            case ParameterType.FromHeader:
                pkValue = req.headers[field.toLowerCase()];
                break;
            case ParameterType.FromParams:
            default:
                pkValue = req.params[field];
                break;
        }
        return pkValue;
    }
    fromDbModelDefaultQueryFunction(callData, _args, param, req) {
        const pkValue = this._extractValue(param, req);
        const query = param.RuntimeType['query']();
        const descriptor = extractModelDescriptor(param.RuntimeType);
        const queryField = param?.Options?.queryField ?? descriptor.PrimaryKey;
        query.select('*');
        query.where(queryField, pkValue);
        /**
         * Checks BelongsToRelations
         */
        for (const [, v] of descriptor.Relations) {
            // if its one-to-one relations ( belongsTo)
            // check if we have same field in route param list
            // If exists, we assume that we want parent ( owner of this model )
            if (v.Type === RelationType.One) {
                const args = callData.Payload?.Param?.Args;
                if (args) {
                    const keys = Object.keys(args);
                    const key = keys.find((k) => {
                        return k.toLowerCase() === v.Name.toLowerCase() || k.toLowerCase() === `_${v.Name.toLowerCase()}`;
                    });
                    if (key) {
                        if (callData.Payload.Param.Args[key]) {
                            query.where(v.ForeignKey, callData.Payload.Param.Args[key]);
                        }
                        else {
                            throw new InvalidArgument(`no key for relation ${v.Name} was provided`);
                        }
                    }
                }
            }
        }
        /**
      * Checks include field
      */
        if (param.Options?.noInclude === true) {
            return query.firstOrThrow(new OrmNotFoundException("Resource not found"));
        }
        /**
         * Checks include field
         */
        if (callData.Payload?.Query?.Args?.include || callData.Payload?.Query?.Args?._include) {
            query.populate(callData.Payload.Query.Args.include ?? callData.Payload.Query.Args._include);
        }
        return query.firstOrThrow(new OrmNotFoundException("Resource not found"));
    }
};
__decorate([
    Autoinject(Container),
    __metadata("design:type", Object)
], FromDbModel.prototype, "Container", void 0);
__decorate([
    Autoinject(Orm),
    __metadata("design:type", Orm)
], FromDbModel.prototype, "Orm", void 0);
FromDbModel = __decorate([
    Injectable()
], FromDbModel);
export { FromDbModel };
export class DbModelHydrator extends ArgHydrator {
    async hydrate(input, parameter) {
        if (input === null) {
            throw new OrmException('primary key cannot be null');
        }
        const model = new parameter.RuntimeType();
        model.hydrate(input);
        return model;
    }
}
export function AsModel(field, type) {
    return Route(Parameter('AsDbModel', null, { field, type }));
}
export function FromModel(options) {
    return Route(Parameter('FromDbModel', null, options));
}
let OrmHttpBootstrapper = class OrmHttpBootstrapper extends Bootstrapper {
    async bootstrap() {
        DI.once('di.resolved.Orm', (_, orm) => {
            // set default route parameter hydrator for all loaded models
            orm.Models.forEach((m) => {
                Reflect.defineMetadata('custom:arg_hydrator', { hydrator: DbModelHydrator }, m.type);
            });
            // add custom mixins
            orm.Models.forEach((m) => {
                // tslint:disable-next-line: forin
                for (const mixin in MODEL_STATIC_MIXINS) {
                    m.type[mixin] = MODEL_STATIC_MIXINS[mixin].bind(m.type);
                }
            });
        });
    }
};
OrmHttpBootstrapper = __decorate([
    Injectable(Bootstrapper)
], OrmHttpBootstrapper);
export { OrmHttpBootstrapper };
//# sourceMappingURL=index.js.map