import { OrmException, SelectQueryBuilder, createQuery } from '@spinajs/orm';
import './builders.js';
export const MODEL_STATIC_MIXINS = {
    async filter(filters) {
        const { query } = createQuery(this, SelectQueryBuilder);
        return query.filter(filters);
    },
    filterColumns() {
        const modelDescriptor = this.getModelDescriptor();
        if (!modelDescriptor) {
            throw new OrmException(`Model ${this.constructor.name} has no descriptor`);
        }
        return [...modelDescriptor.FilterableColumns.entries()].map(([key, val]) => {
            return {
                column: key,
                operators: val,
            };
        });
    },
    filterSchema() {
        const modelDescriptor = this.getModelDescriptor();
        if (!modelDescriptor) {
            throw new OrmException(`Model ${this.constructor.name} has no descriptor`);
        }
        return {
            type: 'array',
            items: {
                type: 'object',
                anyOf: [...modelDescriptor.FilterableColumns.entries()].map(([key, val]) => {
                    return {
                        type: 'object',
                        required: ['Column', 'Operator'],
                        properties: {
                            Column: { const: key },
                            Value: { type: ['string', 'integer', 'array'] },
                            Operator: { type: 'string', enum: val },
                        },
                    };
                }),
            },
        };
    },
};
//# sourceMappingURL=model.js.map